/******************************************************************************\
 * Technische Universitaet Darmstadt, Institut fuer Nachrichtentechnik
 * Copyright (c) 2001
 *
 * Author(s):
 *	Volker Fischer
 *
 * Description:
 *	
 *
 ******************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later 
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT 
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more 
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
\******************************************************************************/

#if !defined(DRMPLOT_H__FD6B2345234523453_804E1606C2AC__INCLUDED_)
#define DRMPLOT_H__FD6B2345234523453_804E1606C2AC__INCLUDED_

#include <qwt_plot.h>
#include "../Vector.h"
#include "../Parameter.h"
#include "../DrmReceiver.h"

extern CDRMReceiver	DRMReceiver;


/* Definitions ****************************************************************/
/* number of available color schemes for the plot */
#define NUM_AVL_COLOR_SCHEMES_PLOT				3

/* Define the plot color profiles */
/* BLUEWHITE */
#define BLUEWHITE_MAIN_PEN_COLOR_PLOT			blue
#define BLUEWHITE_MAIN_PEN_COLOR_CONSTELLATION	blue
#define BLUEWHITE_BCKGRD_COLOR_PLOT				white
#define BLUEWHITE_MAIN_GRID_COLOR_PLOT			gray
#define BLUEWHITE_SPEC_LINE1_COLOR_PLOT			red
#define BLUEWHITE_SPEC_LINE2_COLOR_PLOT			black

/* GREENBLACK */
#define GREENBLACK_MAIN_PEN_COLOR_PLOT			green
#define GREENBLACK_MAIN_PEN_COLOR_CONSTELLATION	red
#define GREENBLACK_BCKGRD_COLOR_PLOT			black
#define GREENBLACK_MAIN_GRID_COLOR_PLOT			QColor(128, 0, 0)
#define GREENBLACK_SPEC_LINE1_COLOR_PLOT		yellow
#define GREENBLACK_SPEC_LINE2_COLOR_PLOT		blue

/* BALCKGREY */
#define BLACKGREY_MAIN_PEN_COLOR_PLOT			black
#define BLACKGREY_MAIN_PEN_COLOR_CONSTELLATION	green
#define BLACKGREY_BCKGRD_COLOR_PLOT				gray
#define BLACKGREY_MAIN_GRID_COLOR_PLOT			white
#define BLACKGREY_SPEC_LINE1_COLOR_PLOT			blue
#define BLACKGREY_SPEC_LINE2_COLOR_PLOT			yellow


/* Classes ********************************************************************/
class CDRMPlot : public QwtPlot
{
    Q_OBJECT

public:
	CDRMPlot(QWidget *p = 0, const char *name = 0);
	virtual ~CDRMPlot() {}

	void SetAvIR(CVector<_REAL>& vecrData, CVector<_REAL>& vecrScale, 
				 _REAL rLowerB, _REAL rHigherB,
				 const _REAL rStartGuard, const _REAL rEndGuard, 
				 const _REAL rBeginIR, const _REAL rEndIR);
	void SetTranFct(CVector<_REAL>& vecrData, CVector<_REAL>& vecrScale);
	void SetAudioSpec(CVector<_REAL>& vecrData, CVector<_REAL>& vecrScale);
	void SetPSD(CVector<_REAL>& vecrData, CVector<_REAL>& vecrScale);
	void SetInpSpec(CVector<_REAL>& vecrData, CVector<_REAL>& vecrScale,
					const _REAL rDCFreq, const _REAL rBWCenter = (_REAL) 0.0,
					const _REAL rBWWidth = (_REAL) 0.0);
	void SetFACConst(CVector<_COMPLEX>& veccData);
	void SetSDCConst(CVector<_COMPLEX>& veccData,
					 CParameter::ECodScheme eNewCoSc);
	void SetMSCConst(CVector<_COMPLEX>& veccData,
					 CParameter::ECodScheme eNewCoSc);
	void SetPlotStyle(const int iNewStyleID);

protected:
	void SetData(CVector<_REAL>& vecrData, CVector<_REAL>& vecrScale,
				 const int  size = 0);
	void SetData(CVector<_COMPLEX>& veccData, QColor color, const int  size);
	void SetQAM4Grid();
	void SetQAM16Grid();
	void SetQAM64Grid();

	/* Colors */
	QColor MainPenColorPlot;
	QColor MainPenColorConst;
	QColor MainGridColorPlot;
	QColor SpecLine1ColorPlot;
	QColor SpecLine2ColorPlot;

public slots:
	void OnClicked(const QMouseEvent& e);
};


#endif // DRMPLOT_H__FD6B2345234523453_804E1606C2AC__INCLUDED_
