/******************************************************************************\
 * Technische Universitaet Darmstadt, Institut fuer Nachrichtentechnik
 * Copyright (c) 2001
 *
 * Author(s):
 *	Volker Fischer
 *
 * Description:
 *
 *
 ******************************************************************************
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
\******************************************************************************/

#include "systemevalDlg.h"


systemevalDlg::systemevalDlg(QWidget* parent, const char* name, bool modal,
	WFlags f) : systemevalDlgBase(parent, name, modal, f)
{
	/* Set help text for the controls */
	AddWhatsThisHelp();

#ifdef _WIN32 /* This works only reliable under Windows :-( */
	/* Get window geometry data from DRMReceiver module and apply it */
	const QRect WinGeom(DRMReceiver.GeomSystemEvalDlg.iXPos,
		DRMReceiver.GeomSystemEvalDlg.iYPos,
		DRMReceiver.GeomSystemEvalDlg.iWSize,
		DRMReceiver.GeomSystemEvalDlg.iHSize);

	if (WinGeom.isValid() && !WinGeom.isEmpty() && !WinGeom.isNull())
		setGeometry(WinGeom);
#endif

	/* Init main plot */
	MainPlot->SetPlotStyle(DRMReceiver.iMainPlotColorStyle);
	MainPlot->setMargin(1);

	/* Default chart (at startup) */
	ButtonInpSpec->setOn(TRUE);
	CharType = INPUTSPECTRUM_NO_AV;

	/* Init slider control */
	SliderNoOfIterations->setRange(0, 4);
	SliderNoOfIterations->
		setValue(DRMReceiver.GetMSCMLC()->GetInitNumIterations());
	TextNumOfIterations->setText("MLC: Number of Iterations: " +
		QString().setNum(DRMReceiver.GetMSCMLC()->GetInitNumIterations()));

	/* Init progress bar for SNR */
	ThermoSNR->setRange(0.0, 30.0);
	ThermoSNR->setOrientation(QwtThermo::Vertical, QwtThermo::Left);
	ThermoSNR->setFillColor(QColor(0, 190, 0));

	/* Update times for color LEDs */
	LEDFAC->SetUpdateTime(1500);
	LEDSDC->SetUpdateTime(1500);
	LEDMSC->SetUpdateTime(600);
	LEDFrameSync->SetUpdateTime(600);
	LEDTimeSync->SetUpdateTime(600);
	LEDIOInterface->SetUpdateTime(2000); /* extra long -> red light stays long */

	/* Init parameter for frequency edit for log file */
	iCurFrequency = 0;

	/* Update controls */
	UpdateControls();


	/* Connect controls ----------------------------------------------------- */
	connect(SliderNoOfIterations, SIGNAL(valueChanged(int)),
		this, SLOT(OnSliderIterChange(int)));

	/* Radio buttons */
	connect(RadioButtonTiLinear, SIGNAL(clicked()),
		this, SLOT(OnRadioTimeLinear()));
	connect(RadioButtonTiWiener, SIGNAL(clicked()),
		this, SLOT(OnRadioTimeWiener()));
	connect(RadioButtonFreqLinear, SIGNAL(clicked()),
		this, SLOT(OnRadioFrequencyLinear()));
	connect(RadioButtonFreqDFT, SIGNAL(clicked()),
		this, SLOT(OnRadioFrequencyDft()));
	connect(RadioButtonFreqWiener, SIGNAL(clicked()),
		this, SLOT(OnRadioFrequencyWiener()));
	connect(RadioButtonTiSyncEnergy, SIGNAL(clicked()),
		this, SLOT(OnRadioTiSyncEnergy()));
	connect(RadioButtonTiSyncFirstPeak, SIGNAL(clicked()),
		this, SLOT(OnRadioTiSyncFirstPeak()));

	/* Buttons */
	connect(ButtonAvIR, SIGNAL(clicked()),
		this, SLOT(OnButtonAvIR()));
	connect(ButtonTransFct, SIGNAL(clicked()),
		this, SLOT(OnButtonTransFct()));
	connect(ButtonFACConst, SIGNAL(clicked()),
		this, SLOT(OnButtonFACConst()));
	connect(ButtonSDCConst, SIGNAL(clicked()),
		this, SLOT(OnButtonSDCConst()));
	connect(ButtonMSCConst, SIGNAL(clicked()),
		this, SLOT(OnButtonMSCConst()));
	connect(ButtonPSD, SIGNAL(clicked()),
		this, SLOT(OnButtonPSD()));
	connect(ButtonInpSpec, SIGNAL(clicked()),
		this, SLOT(OnButtonInpSpec()));
	connect(ButtonAudioSpec, SIGNAL(clicked()),
		this, SLOT(OnButtonAudioSpec()));

	connect(buttonOk, SIGNAL(clicked()),
		this, SLOT(accept()));

	/* Check boxes */
	connect(CheckBoxFlipSpec, SIGNAL(clicked()),
		this, SLOT(OnCheckFlipSpectrum()));
	connect(CheckBoxMuteAudio, SIGNAL(clicked()),
		this, SLOT(OnCheckBoxMuteAudio()));
	connect(CheckBoxWriteLog, SIGNAL(clicked()),
		this, SLOT(OnCheckWriteLog()));
	connect(CheckBoxSaveAudioWave, SIGNAL(clicked()),
		this, SLOT(OnCheckSaveAudioWAV()));
	connect(CheckBoxRecFilter, SIGNAL(clicked()),
		this, SLOT(OnCheckRecFilter()));

	/* Timers */
	connect(&Timer, SIGNAL(timeout()),
		this, SLOT(OnTimer()));
	connect(&TimerChart, SIGNAL(timeout()),
		this, SLOT(OnTimerChart()));
	connect(&TimerLogFileLong, SIGNAL(timeout()),
		this, SLOT(OnTimerLogFileLong()));
	connect(&TimerLogFileShort, SIGNAL(timeout()),
		this, SLOT(OnTimerLogFileShort()));
	connect(&TimerLogFileStart, SIGNAL(timeout()),
		this, SLOT(OnTimerLogFileStart()));

	/* Activte real-time timer */
	Timer.start(GUI_CONTROL_UPDATE_TIME);

	/* Activate delayed log file start if necessary (timer is set to shot
	   only once) */
	if (DRMReceiver.GetParameters()->ReceptLog.IsDelLogStart() == TRUE)
	{
		/* One shot timer */
		TimerLogFileStart.start(DRMReceiver.GetParameters()->
			ReceptLog.GetDelLogStart() * 1000 /* ms */, TRUE);
	}

	/* Update window */
	OnTimerChart();
}

systemevalDlg::~systemevalDlg()
{
	/* Set window geometry data in DRMReceiver module */
	QRect WinGeom = geometry();

	DRMReceiver.GeomSystemEvalDlg.iXPos = WinGeom.x();
	DRMReceiver.GeomSystemEvalDlg.iYPos = WinGeom.y();
	DRMReceiver.GeomSystemEvalDlg.iHSize = WinGeom.height();
	DRMReceiver.GeomSystemEvalDlg.iWSize = WinGeom.width();
}

void systemevalDlg::UpdateControls()
{
	/* Slider for MLC number of iterations */
	const int iNumIt = DRMReceiver.GetMSCMLC()->GetInitNumIterations();
	if (SliderNoOfIterations->value() != iNumIt)
	{
		/* Update slider and label */
		SliderNoOfIterations->setValue(iNumIt);
		TextNumOfIterations->setText("MLC: Number of Iterations: " +
			QString().setNum(iNumIt));
	}

	/* Update for channel estimation and time sync switches */
	switch (DRMReceiver.GetChanEst()->GetTimeInt())
	{
	case CChannelEstimation::TLINEAR:
		if (!RadioButtonTiLinear->isChecked())
			RadioButtonTiLinear->setChecked(TRUE);
		break;

	case CChannelEstimation::TWIENER:
		if (!RadioButtonTiWiener->isChecked())
			RadioButtonTiWiener->setChecked(TRUE);
		break;
	}

	switch (DRMReceiver.GetChanEst()->GetFreqInt())
	{
	case CChannelEstimation::FLINEAR:
		if (!RadioButtonFreqLinear->isChecked())
			RadioButtonFreqLinear->setChecked(TRUE);
		break;

	case CChannelEstimation::FDFTFILTER:
		if (!RadioButtonFreqDFT->isChecked())
			RadioButtonFreqDFT->setChecked(TRUE);
		break;

	case CChannelEstimation::FWIENER:
		if (!RadioButtonFreqWiener->isChecked())
			RadioButtonFreqWiener->setChecked(TRUE);
		break;
	}

	switch (DRMReceiver.GetChanEst()->GetTimeSyncTrack()->GetTiSyncTracType())
	{
	case CTimeSyncTrack::TSFIRSTPEAK:
		if (!RadioButtonTiSyncFirstPeak->isChecked())
			RadioButtonTiSyncFirstPeak->setChecked(TRUE);
		break;

	case CTimeSyncTrack::TSENERGY:
		if (!RadioButtonTiSyncEnergy->isChecked())
			RadioButtonTiSyncEnergy->setChecked(TRUE);
		break;
	}

	/* Update settings checkbuttons */
	CheckBoxMuteAudio->setChecked(DRMReceiver.GetWriteData()->GetMuteAudio());
	CheckBoxFlipSpec->
		setChecked(DRMReceiver.GetReceiver()->GetFlippedSpectrum());
	CheckBoxSaveAudioWave->
		setChecked(DRMReceiver.GetWriteData()->GetIsWriteWaveFile());

	CheckBoxRecFilter->setChecked(DRMReceiver.GetOFDMDemod()->GetRecFilter());

	/* Update frequency edit control (frequency could be changed by
	   schedule dialog */
	QString strFreq = EdtFrequency->text();
	const int iCurLogFreq =
		DRMReceiver.GetParameters()->ReceptLog.GetFrequency();

	if (iCurLogFreq != iCurFrequency)
	{
		EdtFrequency->setText(QString().setNum(iCurLogFreq));
		iCurFrequency = iCurLogFreq;
	}
}

void systemevalDlg::showEvent(QShowEvent* pEvent)
{
	/* Activte real-time timers when window is shown */
	TimerChart.start(GUI_CONTROL_UPDATE_TIME);

	/* Update window */
	OnTimerChart();

	/* Update controls */
	UpdateControls();
}

void systemevalDlg::hideEvent(QHideEvent* pEvent)
{
	/* Deactivate real-time timers when window is hide to save CPU power */
	TimerChart.stop();
}

void systemevalDlg::SetStatus(int MessID, int iMessPara)
{
	switch(MessID)
	{
	case MS_FAC_CRC:
		LEDFAC->SetLight(iMessPara);
		break;

	case MS_SDC_CRC:
		LEDSDC->SetLight(iMessPara);
		break;

	case MS_MSC_CRC:
		LEDMSC->SetLight(iMessPara);
		break;

	case MS_FRAME_SYNC:
		LEDFrameSync->SetLight(iMessPara);
		break;

	case MS_TIME_SYNC:
		LEDTimeSync->SetLight(iMessPara);
		break;

	case MS_IOINTERFACE:
		LEDIOInterface->SetLight(iMessPara);
		break;

	case MS_RESET_ALL:
		LEDFAC->Reset();
		LEDSDC->Reset();
		LEDMSC->Reset();
		LEDFrameSync->Reset();
		LEDTimeSync->Reset();
		LEDIOInterface->Reset();
		break;
	}
}

void systemevalDlg::OnTimerChart()
{
	CVector<_REAL>		vecrData;
	CVector<_COMPLEX>	veccData;
	CVector<_REAL>		vecrScale;
	_REAL				rLowerBound, rHigherBound;
	_REAL				rStartGuard, rEndGuard;
	_REAL				rPDSBegin, rPDSEnd;

	/* CHART ******************************************************************/
	switch (CharType)
	{
	case AVERAGED_IR:
		/* Get data from module */
		DRMReceiver.GetChanEst()->
			GetAvPoDeSp(vecrData, vecrScale, rLowerBound, rHigherBound,
			rStartGuard, rEndGuard, rPDSBegin, rPDSEnd);

		/* Prepare graph and set data */
		MainPlot->SetAvIR(vecrData, vecrScale, rLowerBound, rHigherBound, 
			rStartGuard, rEndGuard, rPDSBegin, rPDSEnd);
		break;

	case TRANSFERFUNCTION:
		/* Get data from module */
		DRMReceiver.GetChanEst()->GetTransferFunction(vecrData, vecrScale);

		/* Prepare graph and set data */
		MainPlot->SetTranFct(vecrData, vecrScale);
		break;

	case POWER_SPEC_DENSITY:
		/* Get data from module */
		DRMReceiver.GetOFDMDemod()->GetPowDenSpec(vecrData, vecrScale);

		/* Prepare graph and set data */
		MainPlot->SetPSD(vecrData, vecrScale);
		break;

	case INPUTSPECTRUM_NO_AV:
		/* Get data from module */
		DRMReceiver.GetReceiver()->GetInputSpec(vecrData, vecrScale);

		/* Prepare graph and set data */
		MainPlot->SetInpSpec(vecrData, vecrScale,
			DRMReceiver.GetParameters()->GetDCFrequency());
		break;

	case AUDIO_SPECTRUM:
		/* Get data from module */
		DRMReceiver.GetWriteData()->GetAudioSpec(vecrData, vecrScale);

		/* Prepare graph and set data */
		MainPlot->SetAudioSpec(vecrData, vecrScale);
		break;

	case FAC_CONSTELLATION:
		/* Get data vector */
		DRMReceiver.GetFACMLC()->GetVectorSpace(veccData);

		/* Prepare graph and set data */
		MainPlot->SetFACConst(veccData);
		break;

	case SDC_CONSTELLATION:
		/* Get data vector */
		DRMReceiver.GetSDCMLC()->GetVectorSpace(veccData);

		/* Prepare graph and set data */
		MainPlot->SetSDCConst(veccData, 
			DRMReceiver.GetParameters()->eSDCCodingScheme);
		break;

	case MSC_CONSTELLATION:
		/* Get data vector */
		DRMReceiver.GetMSCMLC()->GetVectorSpace(veccData);

		/* Prepare graph and set data */
		MainPlot->SetMSCConst(veccData, 
			DRMReceiver.GetParameters()->eMSCCodingScheme);
		break;
	}
}

void systemevalDlg::OnTimer()
{
	_REAL rSNREstimate;
	_REAL rSigmaEst;
	QString strTextWiener = "Doppler / Delay: \t\n";
	QString strTextFreqOffs = "Sample Frequency Offset: \t\n";

	/* Show SNR if receiver is in tracking mode */
	if (DRMReceiver.GetReceiverState() == CDRMReceiver::AS_WITH_SIGNAL)
	{
		/* Get SNR value and use it if available and valid */
		if (DRMReceiver.GetChanEst()->GetSNREstdB(rSNREstimate))
		{
			TextSNR->setText("<center>SNR<br><b>" + 
				QString().setNum(rSNREstimate, 'f', 1) + " dB</b></center>");

			/* Set SNR for log file */
			DRMReceiver.GetParameters()->ReceptLog.SetSNR(rSNREstimate);
		}
		else
			TextSNR->setText("SNR<br><b>---</b>");

		/* Doppler estimation (assuming Gaussian doppler spectrum) */
		if (DRMReceiver.GetChanEst()->GetSigma(rSigmaEst))
		{
			/* Plot delay and Doppler values */
			TextWiener->setText(strTextWiener +
				QString().setNum(rSigmaEst, 'f', 2) + " Hz / " +
				QString().setNum(
				DRMReceiver.GetChanEst()->GetDelay(), 'f', 2) + " ms");
		}
		else
		{
			/* Plot only delay, Doppler not available */
			TextWiener->setText(strTextWiener + "--- / " + QString().setNum(
				DRMReceiver.GetChanEst()->GetDelay(), 'f', 2) + " ms");
		}

		/* Sample frequency offset estimation */
		TextSampFreqOffset->setText(strTextFreqOffs + QString().
			setNum(DRMReceiver.GetParameters()->
			GetSampFreqEst(), 'f', 2) +	" Hz");
	}
	else
	{
		rSNREstimate = 0;

		TextSNR->setText("SNR<br><b>---</b>");

		TextWiener->setText(strTextWiener + "--- / ---");
		TextSampFreqOffset->setText(strTextFreqOffs + "---");
	}
	ThermoSNR->setValue(rSNREstimate);


#ifdef _DEBUG_
	/* Metric values */
	TextFreqOffset->setText("Metrics [dB]: \t\nMSC: " +
		QString().setNum(
		DRMReceiver.GetMSCMLC()->GetAccMetric(), 'f', 2) +	" / SDC: " +
		QString().setNum(
		DRMReceiver.GetSDCMLC()->GetAccMetric(), 'f', 2) +	" / FAC: " +
		QString().setNum(
		DRMReceiver.GetFACMLC()->GetAccMetric(), 'f', 2) + "\nDC Frequency: " +
		QString().setNum(
		DRMReceiver.GetParameters()->GetDCFrequency(), 'f', 3) + " Hz");
#else
	/* DC frequency */
	TextFreqOffset->setText("DC Frequency of DRM Signal: \t\n" +
		QString().setNum(
		DRMReceiver.GetParameters()->GetDCFrequency(), 'f', 2) + " Hz");
#endif


	/* FAC info static ------------------------------------------------------ */
	QString strFACInfo;

	/* Robustness mode #################### */
	strFACInfo = GetRobModeStr() + " / " + GetSpecOccStr();

	FACDRMModeBWL->setText("DRM Mode / Bandwidth:"); /* Label */
	FACDRMModeBWV->setText(strFACInfo); /* Value */


	/* Interleaver Depth #################### */
	switch (DRMReceiver.GetParameters()->eSymbolInterlMode)
	{
	case CParameter::SI_LONG:
		strFACInfo = "2 s (Long Interleaving)";
		break;

	case CParameter::SI_SHORT:
		strFACInfo = "400 ms (Short Interleaving)";
		break;
	}

	FACInterleaverDepthL->setText("Interleaver Depth:"); /* Label */
	FACInterleaverDepthV->setText(strFACInfo); /* Value */


	/* SDC, MSC mode #################### */
	/* SDC */
	switch (DRMReceiver.GetParameters()->eSDCCodingScheme)
	{
	case CParameter::CS_1_SM:
		strFACInfo = "4-QAM / ";
		break;

	case CParameter::CS_2_SM:
		strFACInfo = "16-QAM / ";
		break;
	}

	/* MSC */
	switch (DRMReceiver.GetParameters()->eMSCCodingScheme)
	{
	case CParameter::CS_2_SM:
		strFACInfo += "SM 16-QAM";
		break;

	case CParameter::CS_3_SM:
		strFACInfo += "SM 64-QAM";
		break;

	case CParameter::CS_3_HMSYM:
		strFACInfo += "HMsym 64-QAM";
		break;

	case CParameter::CS_3_HMMIX:
		strFACInfo += "HMmix 64-QAM";
		break;
	}

	FACSDCMSCModeL->setText("SDC / MSC Mode:"); /* Label */
	FACSDCMSCModeV->setText(strFACInfo); /* Value */


	/* Code rates #################### */
	strFACInfo = QString().setNum(DRMReceiver.GetParameters()->MSCPrLe.iPartB);
	strFACInfo += " / ";
	strFACInfo += QString().setNum(DRMReceiver.GetParameters()->MSCPrLe.iPartA);

	FACCodeRateL->setText("Prot. Level (B / A):"); /* Label */
	FACCodeRateV->setText(strFACInfo); /* Value */


	/* Number of services #################### */
	strFACInfo = "Audio: ";
	strFACInfo += QString().setNum(DRMReceiver.GetParameters()->iNumAudioService);
	strFACInfo += " / Data: ";
	strFACInfo +=QString().setNum(DRMReceiver.GetParameters()->iNumDataService);

	FACNumServicesL->setText("Number of Services:"); /* Label */
	FACNumServicesV->setText(strFACInfo); /* Value */


	/* Time, date #################### */
	if ((DRMReceiver.GetParameters()->iUTCHour == 0) &&
		(DRMReceiver.GetParameters()->iUTCMin == 0) &&
		(DRMReceiver.GetParameters()->iDay == 0) &&
		(DRMReceiver.GetParameters()->iMonth == 0) &&
		(DRMReceiver.GetParameters()->iYear == 0))
	{
		/* No time service available */
		strFACInfo = "Service not available";
	}
	else
	{
#ifdef GUI_QT_DATE_TIME_TYPE
		/* QT type of displaying date and time */
		QDateTime DateTime;
		DateTime.setDate(QDate(DRMReceiver.GetParameters()->iYear,
			DRMReceiver.GetParameters()->iMonth,
			DRMReceiver.GetParameters()->iDay));
		DateTime.setTime(QTime(DRMReceiver.GetParameters()->iUTCHour,
			DRMReceiver.GetParameters()->iUTCMin));

		strFACInfo = DateTime.toString();
#else
		/* Set time and date */
		QString strMin;
		const int iMin = DRMReceiver.GetParameters()->iUTCMin;

		/* Add leading zero to number smaller than 10 */
		if (iMin < 10)
			strMin = "0";
		else
			strMin = "";
	
		strMin += QString().setNum(iMin);

		strFACInfo =
			/* Time */
			QString().setNum(DRMReceiver.GetParameters()->iUTCHour) + ":" +
			strMin + "  -  " +
			/* Date */
			QString().setNum(DRMReceiver.GetParameters()->iMonth) + "/" +
			QString().setNum(DRMReceiver.GetParameters()->iDay) + "/" +
			QString().setNum(DRMReceiver.GetParameters()->iYear);
#endif
	}

	FACTimeDateL->setText("Received time - date:"); /* Label */
	FACTimeDateV->setText(strFACInfo); /* Value */


	/* Update controls */
	UpdateControls();
}

void systemevalDlg::OnRadioTimeLinear() 
{
	if (DRMReceiver.GetChanEst()->GetTimeInt() != CChannelEstimation::TLINEAR)
		DRMReceiver.GetChanEst()->SetTimeInt(CChannelEstimation::TLINEAR);
}

void systemevalDlg::OnRadioTimeWiener() 
{
	if (DRMReceiver.GetChanEst()->GetTimeInt() != CChannelEstimation::TWIENER)
		DRMReceiver.GetChanEst()->SetTimeInt(CChannelEstimation::TWIENER);
}

void systemevalDlg::OnRadioFrequencyLinear() 
{
	if (DRMReceiver.GetChanEst()->GetFreqInt() != CChannelEstimation::FLINEAR)
		DRMReceiver.GetChanEst()->SetFreqInt(CChannelEstimation::FLINEAR);
}

void systemevalDlg::OnRadioFrequencyDft() 
{
	if (DRMReceiver.GetChanEst()->GetFreqInt() != CChannelEstimation::FDFTFILTER)
		DRMReceiver.GetChanEst()->SetFreqInt(CChannelEstimation::FDFTFILTER);
}

void systemevalDlg::OnRadioFrequencyWiener() 
{
	if (DRMReceiver.GetChanEst()->GetFreqInt() != CChannelEstimation::FWIENER)
		DRMReceiver.GetChanEst()->SetFreqInt(CChannelEstimation::FWIENER);
}

void systemevalDlg::OnRadioTiSyncFirstPeak() 
{
	if (DRMReceiver.GetChanEst()->GetTimeSyncTrack()->GetTiSyncTracType() != 
		CTimeSyncTrack::TSFIRSTPEAK)
	{
		DRMReceiver.GetChanEst()->GetTimeSyncTrack()->
			SetTiSyncTracType(CTimeSyncTrack::TSFIRSTPEAK);
	}
}

void systemevalDlg::OnRadioTiSyncEnergy() 
{
	if (DRMReceiver.GetChanEst()->GetTimeSyncTrack()->GetTiSyncTracType() != 
		CTimeSyncTrack::TSENERGY)
	{
		DRMReceiver.GetChanEst()->GetTimeSyncTrack()->
			SetTiSyncTracType(CTimeSyncTrack::TSENERGY);
	}
}

void systemevalDlg::OnSliderIterChange(int value)
{
	/* Set new value in working thread module */
	DRMReceiver.GetMSCMLC()->SetNumIterations(value);

	/* Show the new value in the label control */
	TextNumOfIterations->setText("MLC: Number of Iterations: " +
		QString().setNum(value));
}

void systemevalDlg::OnButtonAvIR()
{
	/* Set all other buttons up */
	OnlyThisButDown(ButtonAvIR);

	CharType = AVERAGED_IR;
}

void systemevalDlg::OnButtonTransFct()
{
	/* Set all other buttons up */
	OnlyThisButDown(ButtonTransFct);

	CharType = TRANSFERFUNCTION;
}

void systemevalDlg::OnButtonFACConst()
{
	/* Set all other buttons up */
	OnlyThisButDown(ButtonFACConst);

	CharType = FAC_CONSTELLATION;
}

void systemevalDlg::OnButtonSDCConst()
{
	/* Set all other buttons up */
	OnlyThisButDown(ButtonSDCConst);

	CharType = SDC_CONSTELLATION;
}

void systemevalDlg::OnButtonMSCConst()
{
	/* Set all other buttons up */
	OnlyThisButDown(ButtonMSCConst);

	CharType = MSC_CONSTELLATION;
}

void systemevalDlg::OnButtonPSD()
{
	/* Set all other buttons up */
	OnlyThisButDown(ButtonPSD);

	CharType = POWER_SPEC_DENSITY;
}

void systemevalDlg::OnButtonInpSpec()
{
	/* Set all other buttons up */
	OnlyThisButDown(ButtonInpSpec);

	CharType = INPUTSPECTRUM_NO_AV;
}

void systemevalDlg::OnButtonAudioSpec()
{
	/* Set all other buttons up */
	OnlyThisButDown(ButtonAudioSpec);

	CharType = AUDIO_SPECTRUM;
}

void systemevalDlg::OnlyThisButDown(QPushButton* pButton)
{
	/* Set all buttons of the chart group up */
	if (ButtonAvIR->isOn()) ButtonAvIR->setOn(FALSE);
	if (ButtonTransFct->isOn()) ButtonTransFct->setOn(FALSE);
	if (ButtonFACConst->isOn()) ButtonFACConst->setOn(FALSE);
	if (ButtonSDCConst->isOn()) ButtonSDCConst->setOn(FALSE);
	if (ButtonMSCConst->isOn()) ButtonMSCConst->setOn(FALSE);
	if (ButtonPSD->isOn()) ButtonPSD->setOn(FALSE);
	if (ButtonInpSpec->isOn()) ButtonInpSpec->setOn(FALSE);
	if (ButtonAudioSpec->isOn()) ButtonAudioSpec->setOn(FALSE);

	/* If button was already down, put him back */
	if (pButton->isOn() == FALSE)
		pButton->setOn(TRUE);
}

void systemevalDlg::OnCheckFlipSpectrum()
{
	/* Set parameter in working thread module */
	DRMReceiver.GetReceiver()->
		SetFlippedSpectrum(CheckBoxFlipSpec->isChecked());
}

void systemevalDlg::OnCheckRecFilter()
{
	/* Set parameter in working thread module */
	DRMReceiver.GetOFDMDemod()->
		SetRecFilter(CheckBoxRecFilter->isChecked());
}

void systemevalDlg::OnCheckBoxMuteAudio()
{
	/* Set parameter in working thread module */
	DRMReceiver.GetWriteData()->MuteAudio(CheckBoxMuteAudio->isChecked());
}

void systemevalDlg::OnCheckSaveAudioWAV()
{
/*
	This code is copied in AnalogDemDlg.cpp. If you do changes here, you should
	apply the changes in the other file, too
*/
	if (CheckBoxSaveAudioWave->isChecked() == TRUE)
	{
		/* Show "save file" dialog */
		QString strFileName =
			QFileDialog::getSaveFileName("DreamOut.wav", "*.wav", this);

		/* Check if user not hit the cancel button */
		if (!strFileName.isNull())
		{
			DRMReceiver.GetWriteData()->
				StartWriteWaveFile(strFileName.latin1());
		}
		else
		{
			/* User hit the cancel button, uncheck the button */
			CheckBoxSaveAudioWave->setChecked(FALSE);
		}
	}
	else
		DRMReceiver.GetWriteData()->StopWriteWaveFile();
}

void systemevalDlg::OnTimerLogFileStart()
{
	/* Start logging (if not already done) */
	if (!CheckBoxWriteLog->isChecked())
	{
		CheckBoxWriteLog->setChecked(TRUE);
		OnCheckWriteLog();
	}
}

void systemevalDlg::OnCheckWriteLog()
{
	if (CheckBoxWriteLog->isChecked())
	{
		/* Activte log file timer for long and short log file */
		TimerLogFileShort.start(60000); /* Every minute (i.e. 60000 ms) */
		TimerLogFileLong.start(1000); /* Every second */

		/* Get frequency from front-end edit control */
		QString strFreq = EdtFrequency->text();
		iCurFrequency = strFreq.toUInt();
		DRMReceiver.GetParameters()->ReceptLog.SetFrequency(iCurFrequency);

		/* Set some other information obout this receiption */
		QString strAddText = "";

		/* Check if receiver does receive a DRM signal */
		if ((DRMReceiver.GetReceiverState() == CDRMReceiver::AS_WITH_SIGNAL) &&
			(DRMReceiver.GetReceiverMode() == CDRMReceiver::RM_DRM))
		{
			/* First get current selected audio service */
			int iCurSelServ =
				DRMReceiver.GetParameters()->GetCurSelAudioService();

			/* Check whether service parameters were not transmitted yet */
			if (DRMReceiver.GetParameters()->Service[iCurSelServ].IsActive())
			{
				strAddText = "Label            ";

				/* Service label (UTF-8 encoded string -> convert) */
				strAddText += QString().fromUtf8(QCString(
					DRMReceiver.GetParameters()->Service[iCurSelServ].
					strLabel.c_str()));

				strAddText += "\nBitrate          ";

				strAddText += QString().setNum(DRMReceiver.GetParameters()->
					GetBitRate(iCurSelServ), 'f', 2) + " kbps";

				strAddText += "\nMode             " + GetRobModeStr();
				strAddText += "\nBandwidth        " + GetSpecOccStr();
			}
		}

		/* Set additional text for log file. Conversion from QString to STL
		   string is needed (done with .latin1() function of QT string) */
		string strTemp = strAddText.latin1();
		DRMReceiver.GetParameters()->ReceptLog.SetAdditText(strTemp);

		/* Activate log file */
		DRMReceiver.GetParameters()->ReceptLog.SetLog(TRUE);
	}
	else
	{
		/* Deactivate log file timer */
		TimerLogFileShort.stop();
		TimerLogFileLong.stop();

		DRMReceiver.GetParameters()->ReceptLog.SetLog(FALSE);
	}
}

void systemevalDlg::OnTimerLogFileShort()
{
	/* Write new parameters in log file (short version) */
	DRMReceiver.GetParameters()->ReceptLog.WriteParameters(FALSE);
}

void systemevalDlg::OnTimerLogFileLong()
{
	/* Write new parameters in log file (long version) */
	DRMReceiver.GetParameters()->ReceptLog.WriteParameters(TRUE);
}

QString	systemevalDlg::GetRobModeStr()
{
	switch (DRMReceiver.GetParameters()->GetWaveMode())
	{
	case RM_ROBUSTNESS_MODE_A:
		return "A";
		break;

	case RM_ROBUSTNESS_MODE_B:
		return "B";
		break;

	case RM_ROBUSTNESS_MODE_C:
		return "C";
		break;

	case RM_ROBUSTNESS_MODE_D:
		return "D";
		break;

	default:
		return "A";
	}
}

QString	systemevalDlg::GetSpecOccStr()
{
	switch (DRMReceiver.GetParameters()->GetSpectrumOccup())
	{
	case SO_0:
		return "4,5 kHz";
		break;

	case SO_1:
		return "5 kHz";
		break;

	case SO_2:
		return "9 kHz";
		break;

	case SO_3:
		return "10 kHz";
		break;

	case SO_4:
		return "18 kHz";
		break;

	case SO_5:
		return "20 kHz";
		break;

	default:
		return "10 kHz";
	}
}

void systemevalDlg::AddWhatsThisHelp()
{
/*
	This text was taken from the only documentation of Dream software
*/
	/* DC Frequency Offset */
	QWhatsThis::add(TextFreqOffset,
		"<b>DC Frequency Offset:</b> This is the estimation of the "
		"DC frequency offset. This offset corresponds to the resulting "
		"sound card intermedia frequency of the front-end. This frequency "
		"is not restricted to a certain value. The only restriction is "
		"that the DRM spectrum must be completely inside the bandwidth "
		"of the sound card.");

	/* Sample Frequency Offset */
	QWhatsThis::add(TextSampFreqOffset,
		"<b>Sample Frequency Offset:</b> This is the estimation of "
		"the sample rate offset between the sound card sample rate of "
		"the local computer and the sample rate of the D / A (digital "
		"to analog) converter in the transmitter. Usually the sample rate "
		"offset is very constant for a given sound card. Therefore it is "
		"useful to inform the Dream software about this value at application "
		"startup to increase the acquisition speed and reliability.");

	/* Doppler / Delay */
	QWhatsThis::add(TextWiener,
		"<b>Doppler / Delay:</b> The Doppler frequency of the channel is estimated "
		"for the Wiener filter design of channel estimation in time direction. "
		"If linear interpolation is set for channel estimation in time "
		"direction, this estimation is not updated. The Doppler frequency is "
		"an indication of how fast the channel varies with time. The higher "
		"the frequency, the faster the channel changes are.<br>The total delay "
		"of the Power Delay Spectrum (PDS) is estimated from the impulse "
		"response estimation derived from the channel estimation. This delay "
		"corresponds to the range between the two vertical dashed black lines "
		"in the Impulse Response (IR) plot.");

	/* I / O Interface LED */
	const QString strLEDIOInterface =
		"<b>I / O Interface LED:</b> This LED shows the current status of the "
		"sound card interface. The yellow light shows that the audio output "
		"was corrected. Since the sample rate of the transmitter and local "
		"computer are different, from time to time the audio buffers will "
		"overflow or under run and a correction is necessary. When a "
		"correction occurs, a \"click\" sound can be heard. The red light "
		"shows that a buffer was lost in the sound card input stream. This can "
		"happen if a thread with a higher priority is at 100% and the Dream "
		"software cannot read the provided blocks fast enough. In this case, "
		"the Dream software will instantly loose the synchronization and has "
		"to re-synchronize. Another reason for red light is that the processor "
		"is to slow for running the Dream software.";

	QWhatsThis::add(TextLabelLEDIOInterface, strLEDIOInterface);
	QWhatsThis::add(LEDIOInterface, strLEDIOInterface);

	/* Time Sync Acq LED */
	const QString strLEDTimeSyncAcq =
		"<b>Time Sync Acq LED:</b> This LED shows the state of the timing "
		"acquisition (search for the beginning of an OFDM symbol). If the "
		"acquisition is done, this LED will stay green.";

	QWhatsThis::add(TextLabelLEDTimeSyncAcq, strLEDTimeSyncAcq);
	QWhatsThis::add(LEDTimeSync, strLEDTimeSyncAcq);

	/* Frame Sync LED */
	const QString strLEDFrameSync =
		"<b>Frame Sync LED:</b> The DRM frame synchronization status is shown "
		"with this LED. This LED is also only active during acquisition state "
		"of the Dream receiver. In tracking mode, this LED is always green.";

	QWhatsThis::add(TextLabelLEDFrameSync, strLEDFrameSync);
	QWhatsThis::add(LEDFrameSync, strLEDFrameSync);

	/* FAC CRC LED */
	const QString strLEDFACCRC =
		"<b>FAC CRC LED:</b> This LED shows the Cyclic Redundancy Check (CRC) "
		"of the Fast Access Channel (FAC) of DRM. FAC is one of the three "
		"logical channels and is always modulated with a 4-QAM. If the FAC CRC "
		"check was successful, the receiver changes to tracking mode. The FAC "
		"LED is the indication whether the receiver is synchronized to a DRM "
		"transmission or not.";

	QWhatsThis::add(TextLabelLEDFACCRC, strLEDFACCRC);
	QWhatsThis::add(LEDFAC, strLEDFACCRC);

	/* SDC CRC LED */
	const QString strLEDSDCCRC =
		"<b>SDC CRC LED:</b> This LED shows the CRC check result of the "
		"Service Description Channel (SDC) which is one logical channel of the "
		"DRM stream. This data is transmitted in approx. 1 second intervals "
		"and contains information about station label, audio and data format, "
		"etc. The error protection is normally lower than the protection of "
		"the FAC. Therefore this LED will turn to red earlier than the FAC LED "
		"in general.";

	QWhatsThis::add(TextLabelLEDSDCCRC, strLEDSDCCRC);
	QWhatsThis::add(LEDSDC, strLEDSDCCRC);

	/* MSC CRC LED */
	const QString strLEDMSCCRC =
		"<b>MSC CRC LED:</b> This LED shows the status of the Main Service "
		"Channel (MSC). This channel contains the actual audio and data bits. "
		"The LED shows the CRC check of the AAC core decoder. The SBR has a "
		"separate CRC, but this status is not shown with this LED. If SBR CRC "
		"is wrong but the AAC CRC is ok one can still hear something (of "
		"course, the high frequencies are not there in this case). If this "
		"LED turns red, interruptions of the audio are heard. The yellow light "
		"shows that only one 40 ms audio frame CRC was wrong. This causes "
		"usually no hearable artifacts.";

	QWhatsThis::add(TextLabelLEDMSCCRC, strLEDMSCCRC);
	QWhatsThis::add(LEDMSC, strLEDMSCCRC);

	/* MLC, Number of Iterations */
	const QString strNumOfIterations =
		"<b>MLC, Number of Iterations:</b> In DRM, a multilevel channel coder "
		"is used. With this code it is possible to iterate the decoding "
		"process in the decoder to improve the decoding result. The more "
		"iterations are used the better the result will be. But switching to "
		"more iterations will increase the CPU load. Simulations showed that "
		"the first iteration (number of iterations = 1) gives the most "
		"improvement (approx. 1.5 dB at a BER of 10-4 on a Gaussian channel, "
		"Mode A, 10 kHz bandwidth). The improvement of the second iteration "
		"will be as small as 0.3 dB.<br>The recommended number of iterations "
		"given in the DRM standard is one iteration "
		"(number of iterations = 1).";

	QWhatsThis::add(TextNumOfIterations, strNumOfIterations);
	QWhatsThis::add(SliderNoOfIterations, strNumOfIterations);

	/* Flip Input Spectrum */
	QWhatsThis::add(CheckBoxFlipSpec,
		"<b>Flip Input Spectrum:</b> Checking this box will flip or invert the "
		"input spectrum. This is necessary if the mixer in the front-end uses "
		"the lower side band.");

	/* Mute Audio */
	QWhatsThis::add(CheckBoxMuteAudio,
		"<b>Mute Audio:</b> The audio can be muted by checking this box. "
		"The reaction of checking or unchecking this box is delayed by "
		"approx. 1 second due to the audio buffers.");

	/* Log File */
	QWhatsThis::add(CheckBoxWriteLog,
		"<b>Log File:</b> Checking this box brings the Dream software to write "
		"a log file about the current reception. Every minute the average SNR, "
		"number of correct decoded FAC and number of correct decoded MSC "
		"blocks are logged including some additional information, e.g. the "
		"station label and bit-rate. The log mechanism works only for audio "
		"services using AAC source coding. "
#ifdef _WIN32
		"During the logging no Dream windows "
		"should be moved or re-sized. This can lead to incorrect log files "
		"(problem with QT timer implementation under Windows). This problem "
		"does not exist in the Linux version of Dream."
#endif
		"<br>The log file will be "
		"written in the directory were the Dream application was started and "
		"the name of this file is always DreamLog.txt");

	/* Freq */
	QWhatsThis::add(EdtFrequency,
		"<b>Freq:</b> In this edit control, the current selected frequency "
		"on the front-end can be specified. This frequency will be written "
		"into the log file.");

	/* Wiener */
	const QString strWienerChanEst =
		"<b>Channel Estimation Settings:</b> With these settings, the channel "
		"estimation method in time and frequency direction can be selected. "
		"The default values use the most powerful algorithms. For more "
		"detailed information about the estimation algorithms there are a lot "
		"of papers and books available.<br>"
		"<b>Wiener:</b> Wiener interpolation method uses estimation of the "
		"statistics of the channel to design an optimal filter for noise "
		"reduction.";

	QWhatsThis::add(RadioButtonFreqWiener, strWienerChanEst);
	QWhatsThis::add(RadioButtonTiWiener, strWienerChanEst);

	/* Linear */
	const QString strLinearChanEst =
		"<b>Channel Estimation Settings:</b> With these settings, the channel "
		"estimation method in time and frequency direction can be selected. "
		"The default values use the most powerful algorithms. For more "
		"detailed information about the estimation algorithms there are a lot "
		"of papers and books available.<br>"
		"<b>Linear:</b> Simple linear interpolation method to get the "
		"channel estimate. The real and imaginary parts of the estimated "
		"channel at the pilot positions are linearly interpolated. This "
		"algorithm causes the lowest CPU load but performs much worse than "
		"the Wiener interpolation at low SNRs.";

	QWhatsThis::add(RadioButtonFreqLinear, strLinearChanEst);
	QWhatsThis::add(RadioButtonTiLinear, strLinearChanEst);

	/* DFT Zero Pad */
	QWhatsThis::add(RadioButtonFreqDFT,
		"<b>Channel Estimation Settings:</b> With these settings, the channel "
		"estimation method in time and frequency direction can be selected. "
		"The default values use the most powerful algorithms. For more "
		"detailed information about the estimation algorithms there are a lot "
		"of papers and books available.<br>"
		"<b>DFT Zero Pad:</b> Channel estimation method for the frequency "
		"direction using Discrete Fourier Transformation (DFT) to transform "
		"the channel estimation at the pilot positions to the time domain. "
		"There, a zero padding is applied to get a higher resolution in the "
		"frequency domain -> estimates at the data cells. This algorithm is "
		"very speed efficient but has problems at the edges of the OFDM "
		"spectrum due to the leakage effect.");

	/* Guard Energy */
	QWhatsThis::add(RadioButtonTiSyncEnergy,
		"<b>Guard Energy:</b> Time synchronization tracking algorithm utilizes "
		"the estimation of the impulse response. This method tries to maximize "
		"the energy in the guard-interval to set the correct timing.");

	/* First Peak */
	QWhatsThis::add(RadioButtonTiSyncFirstPeak,
		"<b>First Peak:</b> This algorithms searches for the first peak in the "
		"estimated impulse response and moves this peak to the beginning of "
		"the guard-interval (timing tracking algorithm).");

	/* SNR */
	const QString strSNREst =
		"<b>SNR:</b> Signal to Noise Ratio (SNR) estimation is plotted as a "
		"bar and as a value.";

	QWhatsThis::add(ThermoSNR, strSNREst);
	QWhatsThis::add(TextSNR, strSNREst);

	/* Input Spectrum */
	QWhatsThis::add(ButtonInpSpec,
		"<b>Input Spectrum:</b> This plot shows the Fast Fourier "
		"Transformation (FFT) of the input signal. This plot is active in "
		"both modes, analog and digital. There is no averaging applied. The "
		"screen shot of the Evaluation Dialog shows the significant shape of "
		"a DRM signal (almost rectangular). The dashed vertical line shows the "
		"estimated DC frequency. This line is very important for the analog AM "
		"demodulation. Each time a new carrier frequency is acquired, the red "
		"line shows the selected AM spectrum. If more than one AM spectrums "
		"are within the sound card frequency range, the strongest signal is "
		"chosen.");

	/* Shifted PSD */
	QWhatsThis::add(ButtonPSD,
		"<b>Shifted PSD:</b> This plot shows the estimated Power Spectrum "
		"Density (PSD) of the input signal. The DC frequency (red dashed "
		"vertical line) is fixed at 6 kHz. If the frequency offset acquisition "
		"was successful, the rectangular DRM spectrum should show up with a "
		"center frequency of 6 kHz. This plot represents the frequency "
		"synchronized OFDM spectrum. If the frequency synchronization was "
		"successful, the useful signal really shows up only inside the actual "
		"DRM bandwidth since the side loops have in this case only energy "
		"between the samples in the frequency domain. On the sample positions "
		"outside the actual DRM spectrum, the DRM signal has zero crossings "
		"because of the orthogonality. Therefore this spectrum represents NOT "
		"the actual spectrum but the \"idealized\" OFDM spectrum.");

	/* Transfer Function */
	QWhatsThis::add(ButtonTransFct,
		"<b>Transfer Function:</b> This plot shows the squared magnitude of "
		"the channel estimation at each sub carrier.");

	/* Impulse Response */
	QWhatsThis::add(ButtonAvIR,
		"<b>Impulse Response:</b> This plot shows the estimated Impulse "
		"Response (IR) of the channel based on the channel estimation. It is "
		"the averaged, Hamming Window weighted Fourier back transformation of "
		"the transfer function. The length of PDS estimation and time "
		"synchronization tracking is based on this function. The two red "
		"dashed vertical lines show the beginning and the end of the "
		"guard-interval. The two black dashed vertical lines show the "
		"estimated beginning and end of the PDS of the channel (derived from "
		"the averaged impulse response estimation). If the \"First Peak\" "
		"timing tracking method is chosen, a bound for peak estimation "
		"(horizontal dashed red line) is shown. Only peaks above this bound "
		"are used for timing estimation.");

	/* FAC, SDC, MSC constellations */
	const QString strFACSDCMSCConst =
		"<b>FAC, SDC, MSC:</b> The plots show the constellations of the FAC, "
		"SDC and MSC logical channel of the DRM stream. Depending on the "
		"current transmitter settings, the SDC and MSC can have 4-QAM, 16-QAM "
		"or 64-QAM modulation.";

	QWhatsThis::add(ButtonFACConst, strFACSDCMSCConst);
	QWhatsThis::add(ButtonSDCConst, strFACSDCMSCConst);
	QWhatsThis::add(ButtonMSCConst, strFACSDCMSCConst);

	/* DRM Mode / Bandwidth */
	const QString strRobustnessMode =
		"<b>DRM Mode / Bandwidth:</b> In a DRM system, four possible "
		"robustness modes are defined to adapt the system to different "
		"channel conditions. According to the DRM standard:<ul>"
		"<li><i>Mode A:</i> Gaussian channels, with minor fading</li>"
		"<li><i>Mode B:</i> Time and frequency selective channels, with longer "
		"delay spread</li>"
		"<li><i>Mode C:</i> As robustness mode B, but with higher Doppler "
		"spread</li>"
		"<li><i>Mode D:</i> As robustness mode B, but with severe delay and "
		"Doppler spread</li></ul>The bandwith is the gross bandwidth of the "
		"current DRM signal";

	QWhatsThis::add(FACDRMModeBWL, strRobustnessMode);
	QWhatsThis::add(FACDRMModeBWV, strRobustnessMode);

	/* Interleaver Depth */
	const QString strInterleaver =
		"<b>Interleaver Depth:</b> The symbol interleaver depth can be "
		"either short (approx. 400 ms) or long (approx. 2 s). The longer "
		"the interleaver the better the channel decoder can correct errors "
		"from slow fading signals. But the longer the interleaver length "
		"the longer the delay until (after a re-synchronization) audio can "
		"be heard.";

	QWhatsThis::add(FACInterleaverDepthL, strInterleaver);
	QWhatsThis::add(FACInterleaverDepthV, strInterleaver);

	/* SDC / MSC Mode */
	const QString strSDCMSCMode =
		"<b>SDC / MSC Mode:</b> Shows the modulation type of the SDC and "
		"MSC channel. For the MSC channel, some hierarchical modes are "
		"defined which can provide a very strong protected service channel.";

	QWhatsThis::add(FACSDCMSCModeL, strSDCMSCMode);
	QWhatsThis::add(FACSDCMSCModeV, strSDCMSCMode);

	/* Prot. Level (B/A) */
	const QString strProtLevel =
		"<b>Prot. Level (B/A):</b> The error protection level of the channel "
		"coder. For 64-QAM, there are four protection levels defined in the "
		"DRM standard. Protection level 0 has the highest protection whereas "
		"level 3 has the lowest protection. The letters A and B are the names "
		"of the higher and lower protected parts of a DRM block when Unequal "
		"Error Protection (UEP) is used. If Equal Error Protection (EEP) is "
		"used, only the protection level of part B is valid.";

	QWhatsThis::add(FACCodeRateL, strProtLevel);
	QWhatsThis::add(FACCodeRateV, strProtLevel);

	/* Number of Services */
	const QString strNumServices =
		"<b>Number of Services:</b> This shows the number of audio and data "
		"services transmitted in the DRM stream. The maximum number of streams "
		"is four.";

	QWhatsThis::add(FACNumServicesL, strNumServices);
	QWhatsThis::add(FACNumServicesV, strNumServices);

	/* Received time - date */
	const QString strTimeDate =
		"<b>Received time - date:</b> This label shows the received time "
		"and date in UTC. This information is carried in the SDC channel.";

	QWhatsThis::add(FACTimeDateL, strTimeDate);
	QWhatsThis::add(FACTimeDateV, strTimeDate);

	/* Save audio as wave */
	QWhatsThis::add(CheckBoxSaveAudioWave,
		"<b>Save Audio as WAV:</b> Save the audio signal as stereo, 16-bit, "
		"48 kHz sample rate PCM wave file. Checking this box will let the "
		"user choose a file name for the recording.");

	/* Main plot */
	QWhatsThis::add(MainPlot,
		"<b>Main plot:</b> Graphical display of different vectors of the DRM "
		"decoder. Activate the help text on the selection buttons on the right "
		"to get more information on the specific plot types.");
}
